unit uPainter;

interface

uses
  uHouse                            // for THouse, TWindow, TDoor
  , uPattern                        // for TVisitor
  , graphics                        // for TColor
  ;

type

  TPainter = class(TVisitor)
  private

    FColour: TColor;
    FGotColour: boolean;

    procedure SetColour(Value: TColor);

  public

    procedure AllowVisitorToDoSomethingTo(Visited: TVisited); override;

    property Colour: TColor read FColour write SetColour;

  end;

procedure PaintThing(Thing: TVisited; Colour: TColor);

implementation

procedure PaintThing(Thing: TVisited; Colour: TColor);
var
  Visitor: TPainter;
begin
  Visitor := TPainter.Create;
  Visitor.Colour := Colour;
  Thing.AcceptVisitFrom(Visitor);
end;

{ TPainter }

procedure TPainter.AllowVisitorToDoSomethingTo(Visited: TVisited);
begin
  Assert(FGotColour, 'TPainter - I haven''t got a colour yet');

  if (Visited is THoleStopper) then
  begin

    Assert((THoleStopper(Visited).Material = tmWood), 'TPainter - I can''t paint plastic things');

    if (Visited is TWindow) then
    begin
      TWindow(Visited).Colour := Self.Colour;
      Exit;
    end;

    if (Visited is TDoor) then
    begin
      TDoor(Visited).Colour := Self.Colour;
      Exit;
    end;

  end;

end;

procedure TPainter.SetColour(Value: TColor);
begin
  FColour := Value;
  FGotColour := True;
end;

end.
