unit uSalesman;

interface

uses
  uPattern                        // for TVisitor
  , uHouse                        // for THouse, TWindow, TDoor
  ;

type

  TDoubleGlazingSalesman = class(TVisitor)
  private

    FSomethingIsWood: boolean;        // will be initialised to false

  public

    procedure AllowVisitorToDoSomethingTo(Visited: TVisited); override;
    property SomethingIsWood: boolean read FSomethingIsWood;

  end;

procedure SellToThing(Thing: TVisited);

implementation

uses
  dialogs                       // for MessageDlg
  , uInstaller                  // for TDoubleGlazingInstaller
  , Controls                    // for mrYes
  ;

procedure SellToThing(Thing: TVisited);
var
  Visitor: TDoubleGlazingSalesman;
begin
  Visitor := TDoubleGlazingSalesman.Create;
  try { ... finally }
    Thing.AcceptVisitFrom(Visitor);
    if Visitor.SomethingIsWood then
    begin
      if (MessageDlg('Do you want to replace your wooden bits with plastic?',
                    mtConfirmation, [mbYes, mbNo], 0) = mrYes) then
      begin
        uInstaller.InstallPlasticInThing(Thing);
      end;
    end;
  finally
    Visitor.Free;
  end;  { try ... finally }
end;

{ TDoubleGlazingSalesman }

procedure TDoubleGlazingSalesman.AllowVisitorToDoSomethingTo(
  Visited: TVisited);
begin
  if (Visited is THoleStopper) then
  begin
    // Is there some wood that could be replaced with plastic?
    FSomethingIsWood := THoleStopper(Visited).Material = tmWood;
  end;
end;

end.
 