unit uPainter;

interface

uses
  uHouse                            // for THouse, TWindow, TDoor
  , graphics                        // for TColor
  ;

type

  TPainter = class(THouseVisitor)
  private

    FColour: TColor;
    FGotColour: boolean;

    procedure SetColour(Value: TColor);

  public

    procedure AllowVisitorToDoSomethingTo(House: THouse); override;
    procedure AllowVisitorToDoSomethingTo(Window: TWindow); override;
    procedure AllowVisitorToDoSomethingTo(Door: TDoor); override;

    property Colour: TColor read FColour write SetColour;

  end;

procedure PaintHouse(House: THouse; Colour: TColor);

implementation

procedure PaintHouse(House: THouse; Colour: TColor);
var
  Visitor: TPainter;
begin
  Visitor := TPainter.Create;
  Visitor.Colour := Colour;
  House.AcceptVisitFrom(Visitor);
end;

{ TPainter }

procedure TPainter.AllowVisitorToDoSomethingTo(Window: TWindow);
begin
  Assert(FGotColour, 'TPainter - I haven''t got a colour yet');
  Assert((Window.Material = tmWood), 'TPainter - I can''t paint plastic windows');
  Window.Colour := Self.Colour;
end;

procedure TPainter.AllowVisitorToDoSomethingTo(House: THouse);
begin
  // Nothing to do to a house
end;

procedure TPainter.AllowVisitorToDoSomethingTo(Door: TDoor);
begin
  Assert(FGotColour, 'TPainter - I haven''t got a colour yet');
  Assert((Door.Material = tmWood), 'TPainter - I can''t paint plastic doors');
  Door.Colour := Self.Colour;
end;

procedure TPainter.SetColour(Value: TColor);
begin
  FColour := Value;
  FGotColour := True;
end;

end.
